/*
 *  $Id: io_dos.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#ifdef MSDOS

#include <pc.h>
#include <unistd.h>
#include <iostream.h>
#include <iomanip.h>

#include "vc1541/io_dos.h"

#define HAVE_TSC
#include "misc/profile.h"

#include "vc1541/lptdetct.h"

static const int port_address[IO::NR_OF_PORTS] = {
	0x3bc,
	0x378,
	0x278,
	0x268,
	0x27c,
	0x26c
};
static const char *modes[6] = {
	"N/A",
	"SPP",
	"PS/2",
	"EPP",
	"EPPc",
	"ECP"
};
static const char *ecpM[9] = {
	"no ECR found",
	"Standard Mode",
	"Byte Mode",
	"Parallel Port FIFO Mode",
	"ECP FIFO Mode",
	"EPP Mode",
	"Reserved",
	"FIFO Test Mode",
	"Configuration Mode"
};

static void listPorts(void)
{
	int a, b;
	int port;
	lptMode mode;
        int bios_port[4] = {0, 0, 0, 0};

	for(a = 0;a < 4;a++){
		bios_port[a] = BIOSlpt2port(a + 1);
		if(!bios_port[a]) break;
	}

        cout << "scanning ports...\n";
        for (a = 0;a < IO::NR_OF_PORTS;a++) {
        	port = port_address[a];
                mode = LPTmode(port);
                switch (mode) {
                case lptERR:
			cout.form("\t%03xh: *** conflict with an EPP or "
				  "ECP(EPP-Mode) port at %03xh",
                                  port, port - 4);
                        break;
                case lptMIR:
                	cout.form("\t%03xh: *** mirrored port from %03xh",
                        	  port, port - 4);
                        break;
                default:
                	cout.form("\t%03xh: %s", port, modes[mode]);
                        for (b = 0;b < 4;b++) {
                        	if (port == bios_port[b]) {
                                	cout.form(" (LPT%d)",
                                        	  b + 1);
                                }
                        }
                        if (mode == lptEPPc) {
                        	cout.form(", EPP enable control word: %s",
                                	  EPPcontrol(port));
                        }
                        if (mode == lptECP) {
                        	cout.form(", ECP mode: %s",
                                	  ecpM[ECPmode(port)]);
                        }
                        break;
		}
		cout << "\n\n";
	}
}

static int findPort(void)
{
	int a, port;
	lptMode mode;

	for(a = 0;a < IO::NR_OF_PORTS;a++) {

                port = port_address[a];

		mode = LPTmode(port);

                if (mode == lptSPP) {
			outportb(port, 0x00);
			if (!(inportb(port + 1) & 0x08)) {
				// 1. test passed
				outportb(port, 0x01);
				if (inportb(port + 1) & 0x08) {
					// 2. test passed
					outportb(port, 0x00);
					if (!(inportb(port + 1) & 0x08)) {
						// 3. test passed
                                                return port;
					}
				}
                        }
		}
	}
        return 0;
}

IO_DOS::IO_DOS(word_t port, byte_t atn, byte_t data, byte_t clock) :
	IO(port, atn, data, clock)
{
	profile_value p1, p2;
        int mhz1, mhz2;

        if (port == 0) {
        	port = findPort();
                if (port == 0) {
                	cout << "couldn't detect cable.\n";
                        listPorts();
                        sleep(10);
                        exit(1);
                }
                cout << "cable detected at " << setw(3) <<
                	setfill('0') << hex << port << "h.\n";
        }

        _mhz = 0;
#ifdef HAVE_TSC
	cout << "calibrating delay loop" << flush;

        cout << "." << flush;
        sleep(1);

        cout << "." << flush;
	tsc_read(&p1); sleep(1); tsc_read(&p2);
        mhz1 = (p2.ll - p1.ll) / 1000000;

        cout << "." << flush;
	tsc_read(&p1); sleep(2); tsc_read(&p2);
        mhz2 = (p2.ll - p1.ll) / 1000000;

        _mhz = (5 * mhz2 - 4 * mhz1) / 6 + 1;

        cout << " done.\n";
        cout << "processor seems to run at " << dec << _mhz << "MHz.\n";
        //sleep(5);
#endif /* HAVE_TSC */

	_val = inportb(_port) & ~(ATN_BIT | DATA_BIT | CLOCK_BIT | 0x10);
}

void IO_DOS::set(byte_t bits, byte_t val)
{
	outportb(_port, _val = ((~bits & _val) | val));
}

byte_t IO_DOS::get(byte_t bits)
{
	return inportb(_port) & bits;
}

byte_t IO_DOS::wait(byte_t bits, byte_t val)
{
#ifdef HAVE_TSC
	profile_value p1, p2;

	tsc_read(&p1);
	p1.ll += _mhz * 1000000;
	do {
        	if (get(bits) == val) return 0;
		tsc_read(&p2);
	} while (p1.ll > p2.ll);
	_timeout = 1;
        return 1;
#else /* HAVE_TSC */
	byte_t a;
	unsigned long count = 0;

	while (1) {
		a = inportb(_port);
		if ((a & bits) == val) return 0;
		if (count++ > 1000000) {
			_timeout = 1;
			return 1;
		}
	}
#endif /* HAVE_TSC */
}

byte_t IO_DOS::waitEOI(byte_t bits, byte_t val)
{
#ifdef HAVE_TSC
	profile_value p1, p2;

	tsc_read(&p1);
	p1.ll += _mhz * 1000;
	do {
        	if (get(bits) == val) return 0;
		tsc_read(&p2);
	} while (p1.ll > p2.ll);
        return 1;
#else /* HAVE_TSC */
	byte_t a;
	unsigned long count = 0;

	while (1) {
		a = inportb(_port);
		if ((a & bits) == val) return 0;
		if (count++ > 10000) return 1;
	}
#endif /* HAVE_TSC */
}

void IO_DOS::microSleep(word_t val)
{
#ifdef HAVE_TSC
	profile_value p1, p2;

	tsc_read(&p1);
	p1.ll += _mhz * val;
	do {
		tsc_read(&p2);
	} while (p1.ll > p2.ll);
#endif /* HAVE_TSC */
}

#endif /* MSDOS */
